/*
    MakeMKV GUI - Graphics user interface application for MakeMKV

    Copyright (C) 2009-2010 GuinpinSoft inc <makemkvgui@makemkv.com>

    The contents of this file are subject to the Mozilla Public License
    Version 1.1 (the "License"); you may not use this file except in
    compliance with the License. You may obtain a copy of the License at
    http://www.mozilla.org/MPL/

    Software distributed under the License is distributed on an "AS IS"
    basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
    License for the specific language governing rights and limitations
    under the License.

*/
#include "qtapp.h"
#include "mainwnd.h"

QString GetInfo(AP_UiItem* item,AP_ItemAttributeId Id)
{
    const utf16_t* p = item->GetInfo(Id);
    if (p==NULL)
    {
        return QString();
    }
    if (item->GetInfoConst(Id))
    {
        return QStringFromConstUtf16(p);
    } else {
        return QStringFromUtf16(p);
    }
}

static void AppendItem(QString &str,AP_UiItem *pi,AP_ItemAttributeId id,bool Meta=false)
{
    const utf16_t* p;

    if (NULL!=(p=pi->GetInfo(id)))
    {
        append_const(str,AppGetString( id + AP_UIMSG_VITEM_BASE));
        append_const(str,": ");
        append_const(str,p);
        if (Meta && pi->GetInfo(ap_iaMetadataLanguageName))
        {
            append_const(str," (");
            append_const(str,pi->GetInfo(ap_iaMetadataLanguageName));
            append_const(str,")");
        }
        append_const(str,"<br>");
    }
}

static void AppendCommonAttributes(QString &str,AP_UiItem *pi,bool expertMode)
{
    unsigned int flags;

    AppendItem(str,pi,ap_iaType);

    flags =pi->GetInfoNumeric(ap_iaStreamFlags);
    if ((flags&(AP_AVStreamFlag_DirectorsComments|
        AP_AVStreamFlag_AlternateDirectorsComments|
        AP_AVStreamFlag_ForVisuallyImpaired|
        AP_AVStreamFlag_SecondaryAudio|
        AP_AVStreamFlag_CoreAudio|
        AP_AVStreamFlag_ForcedSubtitles))!=0)
    {
        bool first = true;
        append_const(str,AppGetString( ap_iaStreamFlags + AP_UIMSG_VITEM_BASE));
        append_const(str,": ");
        if ((flags&AP_AVStreamFlag_DirectorsComments)!=0)
        {
            if (false==first) append_const(str,",");
            first=false;
            append_const(str,AP_UI_STRING(APP_IFACE_FLAGS_DIRECTORS_COMMENTS));
        }
        if ((flags&AP_AVStreamFlag_AlternateDirectorsComments)!=0)
        {
            if (false==first) append_const(str,",");
            first=false;
            append_const(str,AP_UI_STRING(APP_IFACE_FLAGS_ALT_DIRECTORS_COMMENTS));
        }
        if ((flags&AP_AVStreamFlag_ForVisuallyImpaired)!=0)
        {
            if (false==first) append_const(str,",");
            first=false;
            append_const(str,AP_UI_STRING(APP_IFACE_FLAGS_FOR_VISUALLY_IMPAIRED));
        }
        if ((flags&AP_AVStreamFlag_CoreAudio)!=0)
        {
            if (false==first) append_const(str,",");
            first=false;
            append_const(str,AP_UI_STRING(APP_IFACE_FLAGS_CORE_AUDIO));
        }
        if ((flags&AP_AVStreamFlag_SecondaryAudio)!=0)
        {
            if (false==first) append_const(str,",");
            first=false;
            append_const(str,AP_UI_STRING(APP_IFACE_FLAGS_SECONDARY_AUDIO));
        }
        if ((flags&AP_AVStreamFlag_ForcedSubtitles)!=0)
        {
            if (false==first) append_const(str,",");
            first=false;
            append_const(str,AP_UI_STRING(APP_IFACE_FLAGS_FORCED_SUBTITLES));
        }
        append_const(str,"<br>");
    }

    AppendItem(str,pi,ap_iaDateTime);
    AppendItem(str,pi,ap_iaName,expertMode);
    AppendItem(str,pi,ap_iaSourceFileName);
    AppendItem(str,pi,ap_iaOriginalTitleId);
    AppendItem(str,pi,ap_iaAngleInfo);
    AppendItem(str,pi,ap_iaLangName);
    AppendItem(str,pi,ap_iaDuration);
    AppendItem(str,pi,ap_iaChapterCount);
    AppendItem(str,pi,ap_iaCodecLong);
    AppendItem(str,pi,ap_iaAudioChannelsCount);
    AppendItem(str,pi,ap_iaAudioSampleRate);
    AppendItem(str,pi,ap_iaAudioSampleSize);
    AppendItem(str,pi,ap_iaVideoSize);
    AppendItem(str,pi,ap_iaVideoAspectRatio);
    AppendItem(str,pi,ap_iaVideoFrameRate);
    AppendItem(str,pi,ap_iaSegmentsCount);
    AppendItem(str,pi,ap_iaSegmentsMap);
    AppendItem(str,pi,ap_iaOutputFileName);

}

static const unsigned int TypicalInfoSize = 512;

QString BuildItemInfo(AP_UiItem* Item,bool expertMode)
{
    QString str;
    str.reserve(TypicalInfoSize);

    append_const(str,Item->GetInfo(ap_iaPanelTitle));

    AppendCommonAttributes(str,Item,expertMode);

    if (str.size()>=TypicalInfoSize)
    {
        qDebug("collection info string reallocated, size=%u",((unsigned int)str.size()));
    }
    str.squeeze();
    return str;
}

